import pandas as pd
import plotly.express as px
from nltk.tokenize import wordpunct_tokenize

class BaseCarbone:
    def __init__(self,path,lang = "français"):

        self.lang = lang
        self.data,self._category_cols = self._prepare_data(path,lang)

    @property
    def categories(self):
        return self.data[self._category_cols]

    def _prepare_data(self,path,lang):

        def clean_text_split(text):
            tokens = wordpunct_tokenize(text)
            tokens = [x for x in tokens if len(x) > 2]
            return tokens

        # Reading and filtering columns in other languages
        data = pd.read_csv(path,sep = ";",encoding = "latin1",low_memory = False)
        data = data[[x for x in data.columns if "espagnol" not in x and "anglais" not in x]]

        # Rename columns to more user friendly col names 
        data = data.rename(columns = {
            f"Tags {lang}":"tags",
            "Code de la catégorie":"category",
            f"Nom base {lang}":"name_base",
            f"Nom attribut {lang}":"name_attribute",
            f"Nom frontière {lang}":"name_attribute2",
            "Type de l'élément":"element_type",
            "Statut de l'élément":"element_status",
            "Structure":"structure",
            "Identifiant de l'élément":"element_id",
            "Type Ligne":"row_type",
            f"Unité {lang}":"unit",
            "Localisation géographique":"geography",
            f"Sous-localisation géographique {lang}":"subgeography",
            "Total poste non décomposé":"emissions",
        })

        # Filter archived rows
        data = data.loc[data["element_status"].str.lower().str.contains("valide")]
        data = data.reset_index(drop = True)

        # Clean text fields, concatenate and propertly tokenize for indexation
        data["name_base"] = data["name_base"].str.replace('"',"").str.strip()
        data["tags"] = data["tags"].str.replace('"',"").str.strip()
        data["text"] = data[["name_base","name_attribute","tags","category"]].apply(lambda x : " ".join(x.dropna()),axis = 1).str.lower()
        data["text_split"] = data["text"].map(clean_text_split)
        data["name"] = data[["name_base","name_attribute"]].apply(lambda x : " ".join(x.dropna()),axis = 1)
        data["full_name"] = data[["name_base","name_attribute","name_attribute2"]].apply(lambda x : " ".join(x.dropna()),axis = 1)
        data["emissions_clean"] = data[["unit","emissions"]].apply(lambda x : f"{x['emissions']} ({x['unit']})",axis = 1)

        # Convert emissions to numeric
        def convert_to_num(x):
            try:
                return float(x.replace(",","."))
            except:
                return x

        data["emissions"] = data["emissions"].map(convert_to_num)

        # Add categories to the columns
        categories = (data["category"]
            .str.split(" > ")
            .apply(pd.Series)
        )
        category_cols = [f"category{i+1}" for i in range(len(categories.columns))]
        categories.columns = category_cols
        data = pd.concat([data,categories],axis = 1)

        return data,category_cols

    def show_data(self,data = None,kind = "treemap",detailed_path = False,color_by_emissions=True,**kwargs):

        assert kind in ["treemap","sunburst","icicle"]

        # Take all data if not provided
        if data is None:
            data = self.data

        # Remove category cols with all NaNs
        all_nans = data[self._category_cols].isnull().sum() == len(data)
        all_nans = all_nans[all_nans].index.tolist()
        data = data.drop(columns = all_nans)
        category_cols = [x for x in self._category_cols if x not in all_nans]

        # Fill NaN for visualization
        data = data.fillna(" ")

        if detailed_path:
            path = [px.Constant("all")] + category_cols + ["name_base","name_attribute","name_attribute2","geography","subgeography"]
        else:
            path = [px.Constant("all")] + category_cols + ["name_base"]

        params = {
            # "values":"emissions",
            "hover_data":["emissions","unit"],
        }

        if color_by_emissions:
            params.update({
                "color":"emissions",
                "color_continuous_scale":"RdBu_r",
            })

        # Treemap visualization (also called Mondrian)
        if kind == "treemap":
            fig = px.treemap(data,path = path,maxdepth = 6,**params,**kwargs)
            
            if color_by_emissions:
                fig.update_traces(
                    root_color="lightgrey",
                    hovertemplate="<b>%{label}</b> - Count: %{value}<br>Emissions: %{color}"
                )

            return fig

        # Sunburst visualization (circular structure chart)
        elif kind == "sunburst":
            fig = px.sunburst(data,path = path[1:],maxdepth = 4,**params,**kwargs)
            return fig

        # icicle visualization (rectangular structure chart)
        elif kind == "icicle":
            fig = px.icicle(data,path = path[1:],maxdepth = 4,**params,**kwargs)
            fig.update_traces(root_color="lightgrey")
            return fig 


    def search(self,query,kind = None,without_split = True,color_by_emissions = True,**kwargs):

        results = self.data.loc[self.data["text_split"].map(lambda x : query in x)].copy()

        if without_split:
            results = results.query("row_type=='Elément'")
        
        # If no visualization
        if kind is None:
            return results
        else:
            fig = self.show_data(data = results.copy(),kind = kind,detailed_path = True,color_by_emissions = color_by_emissions,**kwargs)
            fig.update_layout(title=f"Base Carbone results for query='{query}'")
            return results,fig
            

    def search_word(self,query):
        return self.data.loc[self.data["text"].str.contains(query)]


    # def show_treemap(self,)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['carbonify']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.4.2,<4.0.0',
 'nltk>=3.6.2,<4.0.0',
 'numpy>=1.21.0,<2.0.0',
 'pandas>=1.3.0,<2.0.0',
 'plotly>=5.1.0,<6.0.0',
 'streamlit>=0.84.1,<0.85.0',
 'tqdm>=4.61.2,<5.0.0']

setup_kwargs = {
    'name': 'carbonify',
    'version': '0.1.1',
    'description': 'Open source library for carbon accounting and Lifecycle analysis',
    'long_description': '# Carbonify\n## Open source library for carbon accounting and Lifecycle analysis\n![](docs/assets/banner_carbonify.png)\n\nManipulating carbon data is complex and requires both climate expertise and the knowledge of the right data source to make valid hypothesis.\n\nThe **Carbonify** python library and tools are aimed to democratize data manipulation of carbon data to facilitate accounting and lifecycle analysis.  \n\n\n!!! warning "Experimental"\n    This library is extremely experimental, under active development and alpha-release\n    Don\'t expect the documentation to be up-to-date or all features to be tested\n    Please contact [us](mailto:theo.alvesdacosta@ekimetrics.com) if you have any question\n\n\n## Features\n### Current features\n- Easy access to Base Carbone by ADEME with data indexing\n- Data visualization and search functionalities to easily find carbon ratios\n\n\n## Installation\n### Install from PyPi\nThe library is available on [PyPi](https://pypi.org/project/carbonify/) via \n```\npip install carbonify\n```\n\n### For developers\n- You can clone the github repo / fork and develop locally\n- Poetry is used for environment management, dependencies and publishing, after clone you can run \n\n```\n# To setup the environment\npoetry install\n\n# To run Jupyter notebook or a python console\npoetry run jupyter notebook\npoetry run python\n```\n\n## Contributors\n- [Ekimetrics](https://ekimetrics.com/)\n\n\n\n## Project Structure\n```\n- carbonify/ # Your python library\n- data/\n    - raw/\n    - processed/\n- docs/\n- tests/                            # Where goes each unitary test in your folder\n- scripts/                          # Where each automation script will go\n- requirements.txt                  # Where you should put the libraries version used in your library\n```\n\n\n## References\n\n### Base Carbone\n- [Base Carbone](https://data.ademe.fr/datasets/base-carbone(r)) by ADEME - [Documentation](https://www.bilans-ges.ademe.fr/fr/accueil/contenu/index/page/presentation/siGras/0)\n- [Agribalyse](https://data.ademe.fr/datasets/agribalyse-synthese) by ADEME - [Documentation](https://doc.agribalyse.fr/documentation/conditions-dusage-des-donnees)\n- https://www.hellocarbo.com/blog/calculer/base-carbone/\n\n### LCA \n- [CarbonFact.co](https://carbonfact.co/)\n- https://www.carbonify.app/products/apple-iphone-12-us-64gb\n\n### EDA components\n- https://blog.streamlit.io/the-streamlit-agraph-component/\n- https://github.com/ChrisChross/streamlit-agraph\n',
    'author': 'Theo Alves Da Costa',
    'author_email': 'theo.alvesdacosta@ekimetrics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
